package weixinpayment

import (
	"path"
	"strings"

	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/api"
	"shylinux.com/x/community/src/gonganxitong"
	"shylinux.com/x/community/src/renzhengshouquan"
	"shylinux.com/x/community/src/renzhengshouquan/external/weixinpayment/model"

	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/services/payments/jsapi"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
)

type weixinpayment struct {
	renzhengshouquan.Table
	order  string `data:"11"`
	fields string `data:"app_id,mch_id,api_key,cert_no,cert_key,cert_file,user_uid"`
	create string `name:"create app_id* mch_id* api_key* cert_no* cert_key*:textarea cert_file*:textarea" role:"worker"`
	remove string `name:"remove" role:"worker"`
	list   string `name:"list auth_uid uid auto" role:"worker"`
}

func (s weixinpayment) List(m *ice.Message, arg ...string) {
	s.ValueList(m, arg).Action().Display("")
	m.PushAction(s.Response)
}
func (s weixinpayment) Request(m *ice.Message, arg ...string) {
	msg := s.Select(m.Spawn(), model.AUTH_UID, s.GetAuthUID(m))
	svc := jsapi.JsapiApiService{Client: s.newClient(m, msg)}
	resp, _, err := svc.PrepayWithRequestPayment(m, jsapi.PrepayRequest{
		Appid:       core.String(msg.Append(model.APP_ID)),
		Mchid:       core.String(msg.Append(model.MCH_ID)),
		OutTradeNo:  core.String(arg[3]),
		Description: core.String(m.Option(model.TITLE)),
		Attach:      core.String(m.Option(model.CONTENT)),
		Amount:      &jsapi.Amount{Total: core.Int64(kit.Int64(m.Option(model.AMOUNT)))},
		Payer:       &jsapi.Payer{Openid: core.String(m.Cmd(api.GONGANXITONG_USER, s.Select, model.UID, m.Option(model.USER_UID)).Append(model.OPEN_ID))},
		NotifyUrl:   core.String(strings.Split(m.MergePodCmd("", arg[0]), "?")[0] + path.Join("/action/", kit.Select(kit.Format(s.Response), arg, 1), path.Join(arg[2:]...))),
	})
	m.Info("weixinpayment %s %#v", kit.Format(resp), err)
	if err, ok := err.(*core.APIError); ok {
		m.Warn(ok, err.Message)
		return
	}
	if !m.Warn(err) {
		kit.For(kit.UnMarshal(kit.Format(resp)), func(k, v string) { m.Push(kit.LowerCapital(k), v) })
	}
}
func (s weixinpayment) Response(m *ice.Message, arg ...string) {
	if m.Warn(m.Option("event_type") != "TRANSACTION.SUCCESS") {
		return
	}
	msg := s.Select(m.Spawn(), model.AUTH_UID, s.GetAuthUID(m))
	svc := jsapi.JsapiApiService{Client: s.newClient(m, msg)}
	resp, result, err := svc.QueryOrderByOutTradeNo(m, jsapi.QueryOrderByOutTradeNoRequest{
		Mchid:      core.String(msg.Append(model.MCH_ID)),
		OutTradeNo: core.String(arg[1]),
	})
	m.Info("weixinpayment %s %s %#v", kit.Format(resp), kit.Format(result), err)
	if err, ok := err.(*core.APIError); ok {
		m.Warn(ok, err.Message)
		return
	}
	if m.Warn(*resp.TradeState != "SUCCESS") {
		return
	}
}

func init() { ice.TeamCtxCmd(weixinpayment{}) }

func init() { gonganxitong.PaymentVendor = weixinpayment{} }

func (s weixinpayment) newClient(m, msg *ice.Message) *core.Client {
	if key, err := utils.LoadPrivateKey(msg.Append(model.CERT_KEY)); m.Warn(err) {
		return nil
	} else if client, err := core.NewClient(m, option.WithWechatPayAutoAuthCipher(msg.Append(model.MCH_ID), msg.Append(model.CERT_NO), key, msg.Append(model.API_KEY))); m.Warn(err) {
		return nil
	} else {
		return client
	}
}
